#integration_list {
  background: white;
  list-style: none;
  margin: 0;
  padding: 0;
}
#integration_list > li {
  padding: 1em;
}
#integration_list > li .fa {
  font-size: 130%;
}
#integration_list > li .integration-configs {
  list-style: none;
  margin: 1em 0 0em 0;
  padding: 0;
}
#integration_list > li .integration-configs li {
  box-sizing: border-box;
  margin: 0;
  padding: 1em 0;
}
#integration_list > li .integration-configs li:first-child {
  border-top: 1px #DDD solid;
}
#integration_list > li .integration-configs li a {
  color: blue;
}
#integration_list > li .integration-configs li .fa {
  width: 16px;
  vertical-align: top;
}
#integration_list > li .integration-configs li .fa-ban {
  color: #AAA;
}
#integration_list > li .integration-configs li .fa-check {
  color: #336633;
}
#integration_list > li .integration-configs li .fa-exclamation-circle {
  color: #E68600;
}
#integration_list > li .integration-details {
  box-sizing: border-box;
  width: 100%;
  padding-left: 58px;
  /* Icon size + margin */
}
#integration_list > li .integration-details,
#integration_list > li .integration-icon {
  display: inline-block;
  vertical-align: top;
}
#integration_list > li .integration-icon {
  margin-right: 1em;
  position: absolute;
}
#integration_list > li .integration-name {
  font-size: 120%;
  font-weight: bold;
  margin-top: 0.5em;
  margin-bottom: 0.5em;
}
#integration_list > li .object-tools {
  float: none;
  margin: 2em 0 0.5em 0;
  padding: 0;
}
#integration_list > li .object-tools li {
  display: inline-block;
  float: none;
  margin: 0 1em 0 0;
}
#integration_list > li .object-tools li a {
  background: none;
  color: blue;
  display: inline;
  float: none;
  font-weight: normal;
  margin: 0;
  padding: 0;
  vertical-align: middle;
}
#integration_list > li .object-tools li .fa {
  color: #444;
  font-size: 140%;
  vertical-align: middle;
  width: 16px;
}
#integration_list > li .object-tools li .integration-no-configs {
  color: #444;
}
#integration_list li {
  list-style: none;
}
.integrations-list-page #content-main {
  float: left;
  margin-bottom: 20px;
  width: 100%;
}
/*# sourceMappingURL=integrations.css.map */