.conditions-field {
  overflow: hidden;
}
.conditions-field .conditions-field-action {
  cursor: pointer;
  float: left;
  display: inline-block;
  margin-right: 0.3em;
  text-align: center;
  width: 20px;
  height: 20px;
}
.conditions-field .conditions-field-action .fa {
  line-height: 20px;
  font-size: 16px;
}
.conditions-field .conditions-field-add-condition {
  background: #F9F9F9;
  clear: both;
  display: inline-block;
  line-height: 20px;
  padding: 0.5em;
  color: #222222;
}
.conditions-field .conditions-field-add-condition .fa {
  color: #00AA00;
}
.conditions-field .conditions-field-choice {
  margin-right: 0.3em;
}
.conditions-field .conditions-field-choice select {
  min-width: 130px;
}
.conditions-field .conditions-field-mode ul {
  margin: 0 0 1em 0;
  padding: 0;
}
.conditions-field .conditions-field-mode ul li {
  display: inline-block;
  margin-left: 1em;
}
.conditions-field .conditions-field-mode ul li input {
  margin: 0;
  vertical-align: baseline;
}
.conditions-field .conditions-field-mode ul li label {
  display: inline-block;
}
.conditions-field .conditions-field-operator {
  margin-right: 0.3em;
}
.conditions-field .conditions-field-operator select {
  min-width: 130px;
}
.conditions-field .conditions-field-rows-container {
  border: 1px #CCC solid;
  border-radius: 6px;
  overflow: hidden;
  /* Don't let backgrounds overlap borders. */
}
.conditions-field .conditions-field-rows {
  background: #F3F3F3;
  margin: 0;
  padding: 0;
}
.conditions-field .conditions-field-row {
  border-bottom: 1px #CCC solid;
  clear: both;
  margin: 0;
  padding: 0.5em;
}
.conditions-field .conditions-field-row select {
  vertical-align: top;
}
.conditions-field .conditions-field-row-delete .fa {
  color: #CC0000;
}
.conditions-field .conditions-field-row-options {
  display: block;
  overflow: hidden;
}
.conditions-field .error-list {
  color: #990000;
  margin: 0 0 0.5em 0;
  padding: 0;
}
/*# sourceMappingURL=conditions.css.map */
a.copyable-text-input-link {
  color: #000;
  font-size: 1.5em;
  margin-left: 0.5em;
  vertical-align: middle;
}
/*# sourceMappingURL=copyable_text_input.css.map */
.list-edit-widget .list-edit-remove-item {
  border: none;
}
.list-edit-widget .list-edit-remove-item img {
  border: none;
}
/*# sourceMappingURL=list_edit.css.map */
/**
 * Defines visual states for the consent field.
 *
 * This will set the color of the field's border and data use icon, along with
 * setting the selected label's font style, depending on the currently-selected
 * state.
 *
 * Args:
 *     state (String):
 *         One of ``allow`` or ``block``.
 */
.privacy-consent-field {
  background: #FFFCEA;
  border: 1px #AB952E Solid;
  border-radius: 6px;
  box-shadow: 1px 1px 4px rgba(0, 0, 0, 0.1);
}
.privacy-consent-field.privacy-consent-field-allow {
  background: white;
  border-color: #008800;
  box-shadow: none;
}
.privacy-consent-field.privacy-consent-field-allow .privacy-consent-field-choice-allow label {
  font-weight: bold;
  text-decoration: none;
}
.privacy-consent-field.privacy-consent-field-allow .privacy-consent-field-data-use-icon {
  color: #008800;
}
.privacy-consent-field.privacy-consent-field-allow .privacy-consent-field-header h2 {
  color: #008800;
}
.privacy-consent-field.privacy-consent-field-block {
  background: white;
  border-color: #CC0000;
  box-shadow: none;
}
.privacy-consent-field.privacy-consent-field-block .privacy-consent-field-choice-block label {
  font-weight: bold;
  text-decoration: none;
}
.privacy-consent-field.privacy-consent-field-block .privacy-consent-field-data-use-icon {
  color: #CC0000;
}
.privacy-consent-field.privacy-consent-field-block .privacy-consent-field-header h2 {
  color: #CC0000;
}
.privacy-consent-field.privacy-consent-field-has-icon .privacy-consent-field-header {
  padding-left: 24px;
}
.privacy-consent-field ~ .privacy-consent-field {
  /*
     * Space out the consent fields when using the AllConsentRequirementFields
     * form field. 1em is a default, and may need to be overridden if consumers
     * have a different standard spacing between fields.
     */
  margin-top: 1em;
}
.privacy-consent-field .privacy-consent-field-body {
  padding: 1.5em;
}
.privacy-consent-field .privacy-consent-field-choices {
  font-size: 120%;
  list-style: none;
  margin: 1.5em 0 0 0;
  padding: 0;
  text-align: center;
}
.privacy-consent-field .privacy-consent-field-choices > span {
  display: inline-block;
  margin: 0;
  padding: 0;
  width: 45%;
}
.privacy-consent-field .privacy-consent-field-choices > span.privacy-consent-field-choice-allow label {
  color: #008800;
}
.privacy-consent-field .privacy-consent-field-choices > span.privacy-consent-field-choice-block label {
  color: #CC0000;
}
.privacy-consent-field .privacy-consent-field-choices > span label {
  cursor: pointer;
  text-decoration: underline;
}
.privacy-consent-field .privacy-consent-field-choices input {
  /*
       * Hide the radio buttons. We'll be showing the selected state using the
       * labels.
       */
  display: none;
}
.privacy-consent-field .privacy-consent-field-data-use,
.privacy-consent-field .privacy-consent-field-intent {
  margin: 1.5em 0;
  text-align: justify;
}
.privacy-consent-field .privacy-consent-field-data-use {
  margin: 1.5em;
}
.privacy-consent-field .privacy-consent-field-data-use p {
  overflow: hidden;
}
.privacy-consent-field .privacy-consent-field-data-use-icon {
  color: #444;
  float: left;
  font-size: 150%;
  margin: 0 0.4em 0.4em 0;
}
.privacy-consent-field .privacy-consent-field-header {
  line-height: 24px;
  text-align: center;
  vertical-align: middle;
}
.privacy-consent-field .privacy-consent-field-header h2 {
  display: inline;
  font-size: 120%;
  margin: 0;
  padding: 0;
}
.privacy-consent-field .privacy-consent-field-header img {
  float: right;
}
/*# sourceMappingURL=privacy.css.map */