#integration_list {
  @inner-margin-vert: 0.5em;

  background: white;
  list-style: none;
  margin: 0;
  padding: 0;

  > li {
    padding: 1em;

    .fa {
      font-size: 130%;
    }

    .integration-configs {
      list-style: none;
      margin: 1em 0 0em 0;
      padding: 0;

      li {
        box-sizing: border-box;
        margin: 0;
        padding: 1em 0;

        &:first-child {
          border-top: 1px #DDD solid;
        }

        a {
          color: blue;
        }

        .fa {
          width: 16px;
          vertical-align: top;
        }

        .fa-ban {
          color: #AAA;
        }

        .fa-check {
          color: #336633;
        }

        .fa-exclamation-circle {
          color: #E68600;
        }
      }
    }

    .integration-details {
      box-sizing: border-box;
      width: 100%;
      padding-left: 48px + 10px;  /* Icon size + margin */
    }

    .integration-details,
    .integration-icon {
      display: inline-block;
      vertical-align: top;
    }

    .integration-icon {
      margin-right: 1em;
      position: absolute;
    }

    .integration-name {
      font-size: 120%;
      font-weight: bold;
      margin-top: @inner-margin-vert;
      margin-bottom: @inner-margin-vert;
    }

    .object-tools {
      float: none;
      margin: 2em 0 @inner-margin-vert 0;
      padding: 0;

      li {
        display: inline-block;
        float: none;
        margin: 0 1em 0 0;

        a {
          background: none;
          color: blue;
          display: inline;
          float: none;
          font-weight: normal;
          margin: 0;
          padding: 0;
          vertical-align: middle;
        }

        .fa {
          color: #444;
          font-size: 140%;
          vertical-align: middle;
          width: 16px;
        }

        .integration-no-configs {
          color: #444;
        }
      }
    }
  }

  li {
    list-style: none;
  }
}

.integrations-list-page {
  #content-main {
    float: left;
    margin-bottom: 20px;
    width: 100%;
  }
}
