fieldset {
  &.hidden {
    display: none;
  }

  .description {
    margin: 10px;

    p {
      margin: 0;
      padding: 0;
    }
  }
}

.disabled-reason {
  color: #D00000;
  font-size: 10px !important;

  a {
    color: #900000;
    text-decoration: underline;
  }
}

form .wide p.disabled-reason {
  padding-left: 38px;
}

form .aligned p.disabled-reason {
  padding-left: 38px;
}

.checkbox-row p.disabled-reason {
  margin-left: 0pt;
  padding-left: 0pt !important;
}
