.avatar-preview {
  box-sizing: border-box;
  border-radius: 50%;
  overflow: hidden;
  height: 48px;
  width: 48px;
}
.avatar-preview.avatar-preview-unset {
  background: #FFF;
  border: 1px #CCC dashed;
}
.avatar-preview img {
  width: 48px;
  height: 48px;
}
.avatar-service-configuration,
.avatar-service-fields {
  display: none;
}
.avatar-file-upload-config {
  border: 2px #CCC dashed;
  display: inline-block;
  margin: 1em 0;
  padding: 1em;
}
.avatar-file-upload-config.drag-hover {
  background: white;
  border-color: #000099;
  content: 'Drop to set your avatar';
}
.avatar-file-upload-config.drag-hover * {
  pointer-events: none;
}
.avatar-file-upload-config.drag-hover .avatar-file-upload-instructions {
  display: none;
}
.avatar-file-upload-config.drag-hover .avatar-file-upload-drop-instructions {
  display: block;
}
.avatar-file-upload-config .avatar-file-upload-drop-instructions {
  display: none;
  text-align: center;
}
.avatar-file-upload-config .avatar-preview {
  cursor: pointer;
  margin: 0 auto 1em auto;
}
/*# sourceMappingURL=avatars.css.map */